/**
 * $Id: Repeat.java,v 1.11 2001/10/19 15:33:08 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph repeat node
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

/** Parsing principles:
 * STARTELEMENT
 * - Tests are Tests
 * - Ins are Feedbacks
 * - currentTransformation is stored (it is in the structure)
 * ENDELEMENT
 * - currentTransformation is poped (can be fetch in the structure)
 */

package fr.fluxmedia.transmorpher.Graph ;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ListIterator;

import fr.fluxmedia.transmorpher.Utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.Files.ReadWrite;

public class Repeat extends CallImpl {

      /** The ports that must be tested for equality at each iteration */
  private PortList tests = null;
    /** The process body of the repeat is made of a process */
    private RepeatProcess process = null;
  /** The iterators changing the variables */
  LinearIndexedStruct iterators = null;

  public Repeat (String n, int t, Process p, int in, int out){
    super(n, null, p, in, out);
    // JE/ To be implemented
    if (t != 0) tests = new PortList(t);
    // This should be the lenght of tests
    iterators = new LinearIndexedStruct();
    process = new RepeatProcess(n,this,in,out);
  }
  
  // declaration of the tests is missing
  public PortList getTests(){
    return tests;
  }

  public void addIterator( Iterator i ){
    iterators.add(i.getName(), i);
  }

  public Iterator getIterator( String s ){
    return (Iterator)iterators.get(s);
  }

  public RepeatProcess getProcessBody(){
    return process;
  }

  public PortList inBuf(){
    return process.inPorts();
  }
  
  public PortList tests(){
    return tests;
  }
  
  public PortList outBuf(){
    return process.outPorts();
  }
  
  public void clearNull() {
    super.clearNull();
    process.clearNull();
  }
  public void clearVisited() {
    super.clearVisited();
    process.clearVisited();
  }

  public void retroNull(PortList out, PortList in) {
      // This should be exactly the same thing as Process
      System.err.println("retroNull not implemented yet in repeat");
  }

  /** Prints the XML expression of the call */
  public void generateXML(){
    System.out.print("      <repeat id=\""+id+"\" ");
    System.out.print("in=\"");
    inPorts.generateXML();
    if ( tests != null ) {
        System.out.print("\" test=\"");
	tests.generateXML();
    }
    System.out.print("\" out=\"");
    outPorts.generateXML();
    System.out.print("\" bin=\"");
    process.inPorts().generateXML();
    System.out.print("\" bout=\"");
    process.outPorts().generateXML();
    System.out.println("\">");
    parameters.generateXMLCalls();
    // print the list of iterators
    for( ListIterator l = iterators.listIterator(); l.hasNext() ; ){
      ((Iterator)l.next()).generateXML();
    } //end for
    // print the list of calls
    process.generateXML();
    System.out.println("      </repeat>");
  } //end proc

  /** Prints the Java code for the execution of the call */
  public void generateJavaCode(ReadWrite file){
   System.out.println("[Repeat]generateJavaCode : not implemented yet");
  }

} //end class
