/**
 * $Id: Flatten.java,v 1.2 2001/09/27 07:24:52 laurent Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph rule node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph.Rules ;

import fr.fluxmedia.transmorpher.Files.ReadWrite;

public class Flatten implements Rule {

  public String match = null;

  public Flatten(String m){
    super();
    match = m;
  }

  /** Prints the XML expression of the rule */
  public void generateXML(){
    System.out.print("    <flatten match=\""+match+"\"/>");
  }

  public void generateXSLTCode(ReadWrite file){
    /**  Flatten specification:
    (A (x) (A y) (z)) is reduced into (A x y z)
     This is applied recursivly, i.e. let T being the Flatten transformations it is defiend by:
     T((A x...y)) -> (A S(x)... S(y))
     T((B x...y)) -> (B T(x)...T(y)): this is default template
     S((A x...y)) -> S(x)...S(y)
           S((B x...y)) -> (B T(x)...T(y))
     T= default mode, S=insideA mode
      */
    file.Writeln(2,"<!-- Flatten for "+match+" (default mode) -->");
    file.Writeln(2,"<xsl:template match=\""+match+"\">");
    file.Writeln(4,"<xsl:copy>");
    file.Writeln(6,"<xsl:apply-templates select=\"./*|./text()\" mode=\"inside"+match+"\">");
    file.Writeln(4,"</xsl:copy>");
    file.Writeln(2,"</xsl:template>");
    file.Writeln("");
    file.Writeln(2,"<xsl:template match=\"*|@*|text()\">");
    file.Writeln(4,"<xsl:copy>");
    file.Writeln(6,"<xsl:apply-templates select=\"*|@*|text()\"/>");
    file.Writeln(4,"</xsl:copy>");
    file.Writeln(2,"</xsl:template>");
    file.Writeln("");

    file.Writeln(2,"<!-- Flatten for "+match+" (inside mode) -->");
    file.Writeln(2,"<xsl:template match=\""+match+"\" mode=\"inside"+match+"\">");
    file.Writeln(4,"<xsl:copy>");
    file.Writeln(6,"<xsl:apply-templates select=\"./*|./text()\" mode=\"inside"+match+"\">");
    file.Writeln(4,"</xsl:copy>");
    file.Writeln(2,"</xsl:template>");
    file.Writeln("");
    file.Writeln(2,"<xsl:template match=\"*|@*|text()\" mode=\"inside"+match+"\">");
    file.Writeln(4,"<xsl:copy>");
    file.Writeln(6,"<xsl:apply-templates select=\"*|text()\">");
    file.Writeln(4,"</xsl:copy>");
    file.Writeln(2,"</xsl:template>");
    file.Writeln("");
  }

  /** Prints the XSLT code inside a ModTag template */
  public void generateInsideXSLTCode(ReadWrite file){
    System.out.println("[Flatten]generateInsideXSLTCode : not yet implemented");
  }

  /** Prints the Java code for the execution of the rule */
  public void generateJavaCode(ReadWrite file){
    System.out.println("[Flatten]generateJavaCode : not yet implemented");
  }

}
