/**
 * $Id: RESubst.java,v 1.5 2001/09/28 08:01:02 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph rule node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph.Rules;

import fr.fluxmedia.transmorpher.Files.ReadWrite;

public class RESubst implements Rule {

  public String match = null;
  public String context = null;
  public String source = null;
  public String target = null;
  public String apply = "all";

  public RESubst(String m, String s, String t){
    super();
    match = m;
    source = s;
    target = t;
  }

  public RESubst(String m, String c, String s, String t, String a){
    this(m, s, t);
    context = c;
    apply = a;
  }

  /** Prints the XML expression of the rule */
  public void generateXML(){
    System.out.print("    <resubst match=\""+match+"\"");
    if ( context != null ) System.out.print(" context=\""+context+"\"");
    System.out.println(" source=\""+source+"\" target=\""+target+"\"/>");
  }

  public void generateXSLTCode(ReadWrite file){
    int arob = match.indexOf("@");
    file.Writeln(2,"<!-- Substituting "+apply+" "+source+" by "+target+" in "+match+" -->");
    file.Write(2,"<xsl:template match =\"");
    if( context != null ) file.Write(context+"/");
    file.Writeln(match+"\">");
    file.Writeln(4,"<xsl:if test=\"function-available('regexp:substitute') and function-available('regexp:substituteAll') \">");
   
    if ( arob > 0 ) file.Writeln(6,"<xsl:attribute name=\""+match.substring(arob+1)+"\">");
    
    if (apply.equals("once")) {
      file.Writeln(8,"<xsl:value-of select=\"regexp:substitute(.,'"+source+"','"+target+"')\"/>");
    } else {
      file.Writeln(8,"<xsl:value-of select=\"regexp:substituteAll(.,'"+source+"','"+target+"')\"/>");
    } //end if
   
    if ( arob > 0 ) file.Writeln(6,"</xsl:attribute>");

    file.Writeln(4,"</xsl:if>");
    file.Writeln(2,"</xsl:template>");
    file.Writeln("");
  }

  /** Prints the XSLT code inside a ModTag template */
  public void generateInsideXSLTCode(ReadWrite file){
    System.out.println("[RESubst]generateInsideXSLTCode : not yet implemented");
  }

  /** Prints the Java code for the execution of the rule */
  public void generateJavaCode(ReadWrite file){
    System.out.println("[RESubst]generateJavaCode : not yet implemented");
  }

}
