/**
 * $Id: Ruleset.java,v 1.15 2001/10/20 07:04:52 laurent Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph processes definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph ;

import java.util.ArrayList;
import java.util.Enumeration;
import java.io.File;

import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.Files.Constants;
import fr.fluxmedia.transmorpher.Graph.Rules.*;

public class Ruleset
  extends TransformationImpl {

    ArrayList rules = null;

  public Ruleset(String n, Transmorpher t){
    super(n,"Ruleset",t,1,1);
    rules = new ArrayList(10);
  }

  public void addRule( Rule r ){
    rules.add((Object)r);
  }

  public void insertRule( Rule r, int i ){
    rules.add(i, (Object)r);
  }

  /** Prints the XML expression of the process */
  public void generateXML(){
    System.out.println("  <ruleset name=\""+name+"\" type=\""+type+"\">");
    parameters.generateXMLDeclarations();
    for( int i = 0; i < rules.size(); i++ ){
      ((Rule)rules.get(i)).generateXML();
    }
    System.out.println("  </ruleset>");
    System.out.println();
  };

  /** Prints the Java code for the execution of the call */
  public void generateJavaCode(ReadWrite file){
    System.out.println("[ruleset]generateJavaCode : not yet implemented");
  }

  /** generate the stylesheet for the Query (in the correct file) */
  public void generateXSLTStylesheet(String reloc){
    //String vPath = getTransmorpher().getFile().getAbsoluteFile().getParent() ;
    String vPath = getTransmorpher().getFile().getParent().replace('\\','/') ;
    if (vPath==null) vPath = ".";
    if( reloc != null ) vPath = reloc;
    //System.out.println("[RuleSEt] vPath = " + vPath);
    generateXSLTCode(new ReadWrite(vPath+"/"+name+".xsl",Constants.WRITE));
    iAttributes.setParameter("file",vPath+"/"+name+".xsl");
  }

  /** Prints the Java code for the execution of the call */
  public void generateXSLTCode(ReadWrite file){
    file.Writeln("<?xml version=\"1.0\"?>");
    file.Writeln("<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\"");
    file.Writeln(" xmlns:regexp=\"xalan://fr.fluxmedia.transmorpher.regexp.RegularExpression\">");
    file.Writeln("");
    file.Writeln("<!-- ************************************************************ -->");
    file.Writeln("<!--        This style sheet was generated by Transmorpher        -->");
    file.Writeln("<!-- ************************************************************ -->");
    file.Writeln("");
    for( Enumeration e = parameters.getNames() ; e.hasMoreElements(); ){
      String key = (String)e.nextElement();
      file.Writeln(2,"<xsl:param "+key+">"+parameters.getParameter(key)+"</xsl:param>");
    }
    file.Writeln("");
    file.Writeln("  <!-- Copying the root and its attributes -->");
    file.Writeln("  <xsl:template match=\"/\">");
    file.Writeln("    <xsl:apply-templates/>");
    file.Writeln("  </xsl:template>");
    file.Writeln("  <xsl:template match=\"@*\">");
    file.Writeln("    <xsl:attribute name=\"{name()}\"><xsl:value-of select=\".\"/>");
    file.Writeln("    </xsl:attribute>");
    file.Writeln("  </xsl:template>");
    file.Writeln("");
    file.Writeln("  <!-- Copying all elements and attributes -->");
    file.Writeln("  <xsl:template match=\"*\">");
    file.Writeln("    <xsl:copy>");
    file.Writeln("      <xsl:apply-templates select=\"*|@*|text()\"/>");
    file.Writeln("    </xsl:copy>");
    file.Writeln("  </xsl:template>");
    file.Writeln("");
    file.Writeln("<!-- ************************************************************ -->");
    file.Writeln("<!--     End of the general section, here begins the stylesheet   -->");
    file.Writeln("<!-- ************************************************************ -->");
    file.Writeln("");
    for( int i = 0; i < rules.size(); i++ ){
      ((Rule)rules.get(i)).generateXSLTCode(file);
    }//end for
    file.Writeln("</xsl:stylesheet>") ;
    file.Close();
  }

    public void exec(){
	System.err.println("Ruleset.exec() : not implemented yet");
    }
   
} //end class
