/**
 * $Id: MultipleMixer.java,v 1.12 2001/12/12 13:11:13 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * This class manage a set of OneMixer Objects
  *
  *
  * @author laurent.tardif@inrialpes.fr
  * @date 10 02 2001
  * @see SAX 2.0 && JDK 1.3
*/

package fr.fluxmedia.transmorpher.TProcessFactory ;

import fr.fluxmedia.transmorpher.Files.ReadWrite;

// Imported SAX CLASSES

import org.xml.sax.SAXException;
import org.xml.sax.ContentHandler;
import org.xml.sax.Attributes;
import org.xml.sax.Locator ;
import java.util.Hashtable;



public final class MultipleMixer
  extends TProcessBasic
  implements OneMixerHandler {

  /** The list of OneMixer elements */
  OneMixer[] iMixerList = null ;
  
  /** The number of the current mixer */
  int iCurrentMixer = 0;
  
  /** The number of mixers */
  int iNbMixer = 0;
  
  /** the content handler */
  ContentHandler iContentHandler = null;
  
  /** The constructor */
  public MultipleMixer (int pNbMixer) {
    iMixerList =new OneMixer[pNbMixer];
    iNbMixer = pNbMixer ;
    for (int i=0; i< pNbMixer; i++) {
      iMixerList[i]= new OneMixer(this);
      
    }
    iMixerList[iCurrentMixer].setCurrent(true);
   // iMixerList[1].setCurrent(true);
  }

  /** return a mixer */
  public final OneMixer getMixer(int p) {
    return iMixerList[p];
  }
  
  /** Set the content handler for this element */
  public final void setContentHandler(ContentHandler pContentHandler ) {
    iContentHandler = pContentHandler ;
    for (int i=0; i< iNbMixer; i++) {
      iMixerList[i].setContentHandler(pContentHandler);
    }
  }
  
  /** get the content handler for this element */
  public final ContentHandler getContentHandler( ) {
    return iContentHandler;
  }
  
  /** Is this the last Mixer? */
  public final boolean isLastMixer(OneMixer m) {
    return ( m == iMixerList[iNbMixer-1]);
  }
  
  /** Is this the first Mixer? */
  public final boolean isFirstMixer(OneMixer m) {
    return ( m == iMixerList[0]);
  }
  
  /** when a OneMixer is finished, we execute the stack of the next mixer, and then , set
      the next mixer active */
  public final void IsFinished() throws SAXException {
     // System.out.println("Le mixer ="+iCurrentMixer+" est fini");
    iMixerList[iCurrentMixer++].setCurrent(false);
    if (iCurrentMixer < iNbMixer) {
      //System.out.println("on va passer au :" + (iCurrentMixer));
      iMixerList[iCurrentMixer].ComputeStack();
      iMixerList[iCurrentMixer].setCurrent(true);
    } else {
      // System.out.println("Plus de mixer");
    } //end if
  } //end IsFinished
  
  public final void IsFinishedDuringStop() {
    System.out.println("[MultipleMixer] IsFinishedDuringStop NOT IMPLEMENTED ");
  }
  
  public final TProcess internalCopy() {
    System.out.println("[MultipleMixer] internal Copy NOT IMPLEMENTED ");
    return null;
  }

  public void generatePort(){
    System.out.println("[MultipleMixer] generatePort : useless call ");
  }


} //end class
