/**
 * $Id: TProcessFactory.java,v 1.20 2001/12/12 13:11:13 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
  *
  * This class allow to build the process description
  * And allow the user to start the resolution
  * @author Laurent Tardif (laurent.tardif@inrialpes.fr)
  * @since jdk 1.2 / SAX 2.0
*/

package fr.fluxmedia.transmorpher.TProcessFactory ;


// Imported JAXP classes

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

// Imported SAX classes

import org.xml.sax.SAXException;

// Imported Java Classes

import java.io.IOException;
import java.io.InputStream;
import java.util.Vector ;
import java.util.Enumeration ;

// Imported Transmorpher Classes

import fr.fluxmedia.transmorpher.Utils.Parameters ;
import fr.fluxmedia.transmorpher.Utils.StringParameters ;
import fr.fluxmedia.transmorpher.Utils.SystemResources;


public final class TProcessFactory {

  static final int NB_READER_RISE = 100;
  static final int INFINITY = 100000 ;
    
  static boolean testSerializer =false;
    
  static boolean testReader =false;
    
  Vector iGeneratorList = new Vector(NB_READER_RISE,NB_READER_RISE/5);
   
  /** The Transformer factory */
  static TransformerFactory tFactory =null;
     
  /** the SAXTransformerFactory */
  static SAXTransformerFactory saxTFactory = null ;
    
  /** Class constructor */
  static {
    tFactory = TransformerFactory.newInstance();
    if (tFactory.getFeature(SAXSource.FEATURE) && tFactory.getFeature(SAXResult.FEATURE)) {
      // Cast the TransformerFactory to SAXTransformerFactory.
      saxTFactory = ((SAXTransformerFactory) tFactory);
    }
  } //end static
    
    
  /** The constructor */
  public TProcessFactory ()
  throws TransformerException, TransformerConfigurationException,
          SAXException, IOException {
  //Do nothing :)
  //Very usefull :))) <== call the class constructor :)))))
  }
    
  /** Return the sax factory */
  static final SAXTransformerFactory getSAXTFactory () {
    return saxTFactory;
  }
    
    
  
  /**
       Create a new generator. Currently, this generator is a TReader.
       In the futur, it will be suitable to create different generator according to the type.
       ps : may be, the paramer must be a ParserAnswer instead of String and String[]
       
  */
  public final TProcess newGenerator(String[] pOut,
                                     Parameters pParam,
                                     StringParameters staticAttributes)
      throws SAXException, IOException{
  
      TProcess vTReader = null;
      //System.out.println("[TprocessFactory] new Generator : 0");
      // JE: I am affraid that this test has been done when constructing the graph... useless
      if((testReader) && (staticAttributes.getParameter("type").equals("standardInput"))) {
	  System.out.println("[newGenerator]error : only one generator can be"
			     + " connected to the standard input");
	  System.exit(1);
      } else {
	  //if (staticAttributes ==null) System.out.println("param nuill");
	  //if (staticAttributes.getParameter("type") ==null) System.out.println("param(type_ nuill2");
	  if(staticAttributes.getParameter("type").equals("standardInput")) testReader=true;
	  //System.out.println("[TprocessFactory] new Generator : " + pParam);
	  try {
	      String ReaderClassName = SystemResources.getProperty("XML_READER");
	      //System.out.println("[TprocessFactory] Creation d'un TReader :"+ReaderClassName);
	      Class myReaderClass =  Class.forName(ReaderClassName);
	      //System.out.println("[TprocessFactory] Creation d'un TReader :"+ReaderClassName);
	      java.lang.reflect.Constructor[] myReaderConstructors = myReaderClass.getConstructors();
	      
	      Object[] myParam = new Object[2];
	      myParam[0] = pOut ;
	      myParam[1] = pParam ;
	      //System.out.println("[TprocessFactory] Creation d'un TReader :"+ReaderClassName);
	      //System.out.println("[TprocessFactory] NbConstructor Public :"+ myReaderConstructors.length);
	      TProcess myReaderInstance = (TProcess) myReaderConstructors[0].newInstance(myParam);
	      vTReader = myReaderInstance;
	  } catch (Exception e) {
	      System.out.println("[TprocessFactory]Error : lors de la creation d'un TReader, "
				 +"on utilse le Reader par defaut");
	      System.out.println("[TprocessFactory]Error : "+e);                          
	      vTReader = new TReader(pOut,pParam);
	  } //end try
      } //end if
      return vTReader;
  } //end proc
    

  /**
    Create a new loop
  
  */
  public final TProcess newLoop(String[] pIn,
                                String[] pOut,
                                String pRefOn,
                                int pCount,
                                TProcessComposite pTemplateList)
  throws TransformerException,
         TransformerConfigurationException,
         SAXException,
         IOException{
     return new TLoop(pIn,pOut,pRefOn,pCount,pTemplateList);
  }
    

    
  /**
      Create a new main process
  */
  public static final TProcess newMain(String[] pIn,
                                       String[] pOut, Parameters p)
      throws TransformerException,
      TransformerConfigurationException,
      SAXException,
      IOException {
      return new TMain(p);
  }
    
    /**
    Create a new servlet (for the moment the same thing than a main
    */
  public static final TProcess newServlet(String[] pIn,
            String[] pOut, Parameters p)
      throws TransformerException,
           TransformerConfigurationException,
      SAXException,
      IOException {
    return new TServlet(p);
  }
   
    
  /**
     Build a new dispatcher. currently only build a basic one. In the next version more complex
     dispatcher can be developed. (With filter, ...)
  */
  public static final TProcess newDispatcher(String[] pIn,
                                             String[] pOut,
                                             Parameters pParam) {
    return new TDispatcher(pIn,pOut,pParam);
  }
    
    
    /**
    Build a new Transformer according th the given type. Currently only
    build  transformer with a XSL file in parameter.
    */
  public static final TProcess newExternal( String[] pIn,
                                            String[] pOut,
                                            Parameters pParam)
      throws TransformerException,TransformerConfigurationException,
             SAXException, IOException  {
    
    return new TApplyExternal(pIn,pOut,pParam);
  }
 
 /**
    Build a new query. currently only build a internal one. In the next version more complex
    query can be developp.
 */
    public static final TProcess newApplyQuery( String[] pIn,
                                                String[] pOut,
                                                Parameters pParam,
                                                StringParameters staticAttributes)
      throws TransformerException,SAXException, IOException  {
  return new TApplyQuery(pIn,pOut,pParam,staticAttributes);
  }
  
 public static final TProcess newApplyProcess( String[] pIn,
                                               String[] pOut,
                                               Parameters pParam,
                                               StringParameters staticAttributes)
      throws TransformerException,TransformerConfigurationException,
             SAXException, IOException  {
    
    return new TApplyProcess(pIn,pOut,pParam,staticAttributes);
  }
 
  /** Build an new ruleset process */
  public static final TProcess newRuleSet(String[] pIn,
                                          String[] pOut,
                                          Parameters pParam,
                                          StringParameters staticAttributes)
  throws TransformerException,
         TransformerConfigurationException, SAXException, IOException  {
    return new TApplyRuleSet(pIn,pOut,pParam,staticAttributes);
  } //end proc
  
    
    /**
       Build a new Connector. currently only build a basic one. In the next version more complex
       Connector can be developed. (With filter, ...)
    */
  public static final TProcess newConnector(String[] pIn,
                                            String[] pOut,
                                            Parameters pParam)  {
    try {
      String ConnectorClassName = SystemResources.getProperty("XML_CONNECTOR");
      //System.out.println("[TprocessFactory] Creation d'un connector :"+ConnectorClassName);
      Class myConnectorClass =  Class.forName(ConnectorClassName);
      //System.out.println("[TprocessFactory] Creation d'un connector :"+ConnectorClassName);
      java.lang.reflect.Constructor[] myConnectorConstructors = myConnectorClass.getConstructors();
    
      Object[] myParam = new Object[3];
      myParam[0] = pIn;
      myParam[1] = pOut ;
      myParam[2] = pParam ;
      //System.out.println("[TprocessFactory] Creation d'un connector :"+ConnectorClassName);
      //System.out.println("[TprocessFactory] NbConstructor Public :"+ myConnectorConstructors.length);
      TProcess myConnectorInstance = (TProcess) myConnectorConstructors[0].newInstance(myParam);
      /*
      java.Lang.Class             forName(String className) 
      Constructor[] getConstructors() 
      java.lang.reflect.Constructor
      Constructor : Object newInstance(Object[] initargs) 
     */
     return myConnectorInstance;
    } catch (Exception e) {
      System.out.println("[TprocessFactory]Error : lors de la creation d'un connector, "
                          +"on utilse le connector par defaut");
      System.out.println("[TprocessFactory]Error : "+e);                          
      return new TConnector(pIn,pOut,pParam);
    }
    
    
    //return new TConnector(pIn,pOut,pParam);
  }
    
    /**
       Build a new Connector. currently only build a basic one. In the next version more complex
       Connector can be developp. (With filter, ...)
    */
    
  public static final TProcess newSerializer(String[] pIn,
                                             Parameters pParam,
                                             StringParameters staticAttributes)
      throws SAXException,IOException {
  
      TProcess serializer = null;
      if( (testSerializer) && (staticAttributes.getParameter("type").equals("standardOuput"))){
	  System.out.println("[newGenerator]error : only one generator can be"
			     +" connected to the standard input");
	  System.exit(1);
      } //end if
      if(staticAttributes.getParameter("type").equals("standardOutput")) testReader=true;
      try {
	  String serializerClassName = SystemResources.getProperty("XML_SERIALIZER");
	  //System.out.println("[TprocessFactory] Creation d'un SERIALIZER :"+serializerClassName);
	  Class mySerializerClass =  Class.forName(serializerClassName);
	  //System.out.println("[TprocessFactory] Creation d'un SERIALIZER :"+serializerClassName);
	  java.lang.reflect.Constructor[] mySerializerConstructors = mySerializerClass.getConstructors();
	  
	  Object[] myParam = new Object[3];
	  myParam[0] = pIn;
	  myParam[1] = pParam ;
	  myParam[2] = staticAttributes;
	  //System.out.println("[TprocessFactory] Creation d'un serializer :"+serializerClassName);
	  //System.out.println("[TprocessFactory] NbConstructor Public :"+ mySerializerConstructors.length);
	  serializer  = (TProcess) mySerializerConstructors[0].newInstance(myParam);
      } catch (Exception e) {
	  System.out.println("[TprocessFactory]Error : lors de la creation d'un serializer, "
			     +"on utilse le serializer par defaut");
	  System.out.println("[TprocessFactory]Error : "+e);                          
	  serializer = new TSerializer(pIn,pParam,staticAttributes);
      } //end catch
    
    
   
    
    return serializer;
  } //end proc

}

