/**
 * $Id: TReader.java,v 1.20 2001/12/12 13:11:13 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
 * This class provide a basic process which is
 * able to read an XML file
 *
 * @author Laurent.Tardif@inrialpes.fr
 * @since jdk 1.3 && SAX 2.0
 */


package fr.fluxmedia.transmorpher.TProcessFactory ;

import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.Utils.SystemResources;

// Imported SAX classes

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;

// Imported java.io classes

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

//For thread

import java.lang.Runnable ;
import fr.fluxmedia.transmorpher.Utils.Parameters ;

public final class TReader
  extends TProcessBasic
  implements Runnable {
    
   //
   // Constants
   //

   // feature ids

   /** Namespaces feature id (http://xml.org/sax/features/namespaces). */
    protected static final String NAMESPACES_FEATURE_ID = "http://xml.org/sax/features/namespaces";
    
   /** Namespaces feature id (http://xml.org/sax/features/namespaces). */
    protected static final String NAMESPACES_PREFIXES_FEATURE_ID = "http://xml.org/sax/features/namespace-prefixes";

   /** Validation feature id (http://xml.org/sax/features/validation). */
   protected static final String VALIDATION_FEATURE_ID = "http://xml.org/sax/features/validation";

   /** Schema validation feature id (http://apache.org/xml/features/validation/schema). */
   protected static final String SCHEMA_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/schema";

  // property ids

  /** Lexical handler property id (http://xml.org/sax/properties/lexical-handler). */
  protected static final String LEXICAL_HANDLER_PROPERTY_ID = "http://xml.org/sax/properties/lexical-handler";

  // default settings

  /** Default parser name. */
  //protected static final String DEFAULT_PARSER_NAME = "org.apache.xerces.parsers.SAXParser";
  
  // default settings

  /** Default namespaces support (true). */
  protected static final boolean DEFAULT_NAMESPACES = true;

  /** Default namespaces support (true). */
  protected static final boolean DEFAULT_NAMESPACES_PREFIXES = true;

  /** Default validation support (false). */
  protected static final boolean DEFAULT_VALIDATION = false;
    
  /** Default Schema validation support (false). */
  protected static final boolean DEFAULT_SCHEMA_VALIDATION = false;

  /** The XMLReader */
  XMLReader iReader = null;
  boolean isThread = false ;
    
  /** The file name whish describe the file to read */
  String iFilename = null;
   
  boolean isRunning = false ;
  Thread myThread = null;
  
  InputSource standardInput = null;
    
  /** the empty constructor */
   /*private TReader(){}


   public TReader(String[] pOut) throws SAXException, IOException{
    super(new String[0],pOut);
  }
    
  public TReader(String[] pOut,String type) throws SAXException, IOException{
    super(new String[0],pOut);
    this.type = type;;
  }
    */
  public TReader (String[] pOut, Parameters pParam)
    throws SAXException, IOException   {

    super(new String[0],pOut);
    setParameters(pParam);
    type = (String) pParam.getParameter("type");
    try {
      //the hypothesis made is the reader is an org.xml.sax.XMLReader
      iReader = XMLReaderFactory.createXMLReader(SystemResources.getProperty("XML_PARSER"));
      
      //iReader = XMLReaderFactory.createXMLReader("fr.fluxmedia.transmorpher.Files.CharEngine");
      iReader.setFeature(NAMESPACES_FEATURE_ID, DEFAULT_NAMESPACES);
      iReader.setFeature(NAMESPACES_PREFIXES_FEATURE_ID, DEFAULT_NAMESPACES_PREFIXES);
      iReader.setFeature(VALIDATION_FEATURE_ID, DEFAULT_VALIDATION);
      iReader.setFeature(SCHEMA_VALIDATION_FEATURE_ID, DEFAULT_SCHEMA_VALIDATION);
      //iReader.setFeature("http://xml.org/sax/features/namespace-prefixes",true);
    } catch(Exception e) {
      System.out.println("[Treader]Exception :"+e);
      System.out.println("[TReader] "+SystemResources.getProperty("XML_READER"));
    } //end try
    if (isThread) myThread = new Thread(this);
  }
               
   
    /** Exec the process */
  public void execProcess() throws IOException,SAXException {
    iFilename = (String)getParameter("file");
    if (isThread) {
      //System.out.println("We hope that the link method has been call (Thread) ") ; //:-)
      myThread.start();
    } else {
      //System.out.println("We hope that the link method has been call (no thread)") ; //:-)
      if(iFilename!=null) {
        //System.out.println("[TReader]filename : " + iFilename + " read by =" +iReader);
        iReader.parse(iFilename);
      } else {
        //System.out.println("standard input : ");
        iReader.parse(standardInput);
      } //end if
      //System.out.println("aprs le parse");
    } //end if
    //System.out.println("Fin de l'ordre de parse") ;
  } //end proc
    

    
  public void generatePort(){
    //System.out.println("iReader "+iReader);
    iListOut[0] = new XML_Port(nameOut[0],this,(XMLReader)iReader,0,XML_Port.OUT);
  }

  public final void run() {
    //System.out.println("euh .. je fais qqc ?");
    if (!isRunning) {
      try {
        //System.out.println("debut du parse  2");
        iReader.parse(iFilename);
        //System.err.println("Fin du parse  2");
        isRunning=true;
        //myThread.interrupt();
      } catch (Exception e) {
        System.err.println("Fin du parse 2 !erreur dans le thread" + e);
        e.printStackTrace();
        isRunning=true;
      } //end try
    } //fin if
  } //end run
}
