/**
 * $Id: TTransformer.java,v 1.14 2001/12/12 13:11:13 jerome Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
  *
  * This class allow to build a Transformer process
  *
  * @author Laurent Tardif (laurent.tardif@inrialpes.fr)
  * @since jdk 1.2 / SAX 2.0
  * @see fr.fluxmedia.transmorpher.TProcessFactory.XML_Port
*/

package fr.fluxmedia.transmorpher.TProcessFactory ;

// Imported JAXP classes

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.sax.SAXResult;

// Imported SAX classes

import org.xml.sax.SAXException;

// Imported java.io classes

import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;

// Imported transmorpher classes

import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.Utils.Parameters;

public  class TTransformer
  extends TProcessBasic{
    
  /** The transformer */
  TransformerHandler iTransformerHandler = null;
    
  /** The XslFile, it's the parameter of the transformation */
  protected String iXSLFile = null;
    
  /** the properties for all transformer */
  static java.util.Properties Op = null ;
    
  /** port name in */
  String[] In = null;
    
  /** port name out */
  String[] Out = null;
    

  /** The instance  constructor
  
  @throws TransformerException
  @throws TransformerConfigurationException
  @throws SAXException
  @throws IOException
  
  */
  

  /*---------------------------- Constructors -----------------------------------------*/

  public TTransformer (String[] pIn, String[] pOut, Parameters p, String filename)
      throws TransformerException, TransformerConfigurationException,
             SAXException, IOException   {
  
    super(pIn,pOut);
    setParameters(p);
    initTransformer(filename);
  
  } //End constructor
 
  /*---------------------------- Initialization -----------------------------------------*/
    // JE: I am pretty sure that this should be achieved at binding-time and not creation time

  /** get The Transformer Handler */
  public final void initTransformer(String xslFile)
  throws TransformerException, TransformerConfigurationException, SAXException, IOException {
    iXSLFile = xslFile;
    //System.out.println("[TTransformer] + file =" + iXSLFile);
    StreamSource SS = new StreamSource(iXSLFile) ;
    // Create a TransformerHandler for each stylesheet.
    iTransformerHandler = TProcessFactory.getSAXTFactory().newTransformerHandler(SS);
    //System.out.println("[TTransformer]creation d'un transformer "
                        //+ iTransformerHandler + "," + iXSLFile);
    setProperties();
  }

  /*------------------------    TransformerHandler getTransfomerHandler -----------------*/
    
  /** get The Transformer Handler */
  public final TransformerHandler getTransformerHandler() {
    return iTransformerHandler;
  }
    
   
  /*------------------------    void  setProperties --------------------------------------*/

  public void setProperties(){
  
    if (Op == null) {
      Op = iTransformerHandler.getTransformer().getOutputProperties();
      Op.setProperty(javax.xml.transform.OutputKeys.ENCODING,"iso-8859-1");
      Op.setProperty(javax.xml.transform.OutputKeys.INDENT,"yes");
      Op.setProperty(javax.xml.transform.OutputKeys.METHOD,"xml");
    } //end if
    iTransformerHandler.getTransformer().setOutputProperties(Op);
  }
    
    
  /*------------------------    void  bindParameters --------------------------------------*/

    public void bindParameters( Parameters p ){
	Transformer trans = iTransformerHandler.getTransformer();
	//System.err.println("In bP@Trans: "+this);
	// First get the name of parameters that must be bound
	p.bindParameters( iListOfParameters );
	// Then pass them to the transformer instance
	for( Enumeration e = iListOfParameters.getNames() ; e.hasMoreElements(); ){
	    String key = (String)e.nextElement();
	    //System.err.println("In TT: Set "+key+" to "+(String)iListOfParameters.getParameter(key));
	    trans.setParameter(key,iListOfParameters.getParameter(key));
	} //end for
    }
    
    
  public void generatePort(){
    
    String nameI = nameIn[0];
    String nameO = nameOut[0];
  
    if(nameFather!=null)  {
      nameI = nameFather+"-"+nameI;
      nameO = nameFather+"-"+nameO;
     }
    iListIn [0] = new XML_Port(nameI,this,iTransformerHandler,0,XML_Port.IN);
    iListOut[0] = new XML_Port(nameO,this,iTransformerHandler,0,XML_Port.OUT);
  }

    // It would be useful to directly feed this Transformer with Templates...
    // This can be found on p24 of current JAXP 1.1 spec

    
}//end class
